@@ -11,7 +11,7 @@ pub fn format(sh: &Shell, flags: flags::Format) -> anyhow::Result<()> {
         .and_then(|_| crate::cargo())
         .context("failed to run task 'format'")?;
 
-    for WorkspaceMember { crate_name, .. } in crate::WORKSPACE_MEMBERS.iter() {
+    for WorkspaceMember { crate_name, .. } in crate::workspace_members().iter() {
         let _pd = sh.push_dir(Path::new(crate_name));
         // Tell the user where we are now
         println!();
