@@ -5,6 +5,7 @@ pub mod tab;
 
 mod background_jobs;
 mod logging_pipe;
+mod pane_groups;
 mod plugins;
 mod pty;
 mod pty_writer;
@@ -15,17 +16,20 @@ mod terminal_bytes;
 mod thread_bus;
 mod ui;
 
+pub use daemonize;
+
 use background_jobs::{background_jobs_main, BackgroundJob};
 use log::info;
+use nix::sys::stat::{umask, Mode};
 use pty_writer::{pty_writer_main, PtyWriteInstruction};
 use std::collections::{BTreeMap, HashMap, HashSet};
 use std::{
+    net::{IpAddr, Ipv4Addr},
     path::PathBuf,
     sync::{Arc, RwLock},
     thread,
 };
 use zellij_utils::envs;
-use zellij_utils::nix::sys::stat::{umask, Mode};
 use zellij_utils::pane_size::Size;
 
 use wasmtime::{Config as WasmtimeConfig, Engine, Strategy};
@@ -44,7 +48,7 @@ use zellij_utils::{
     consts::{
         DEFAULT_SCROLL_BUFFER_SIZE, SCROLL_BUFFER_SIZE, ZELLIJ_SEEN_RELEASE_NOTES_CACHE_FILE,
     },
-    data::{ConnectToSession, Event, InputMode, KeyWithModifier, PluginCapabilities},
+    data::{ConnectToSession, Event, InputMode, KeyWithModifier, PluginCapabilities, WebSharing},
     errors::{prelude::*, ContextType, ErrorInstruction, FatalError, ServerContext},
     home::{default_layout_dir, get_default_data_dir},
     input::{
@@ -58,7 +62,7 @@ use zellij_utils::{
         plugins::PluginAliases,
     },
     ipc::{ClientAttributes, ExitReason, ServerToClientMsg},
-    shared::default_palette,
+    shared::{default_palette, web_server_base_url},
 };
 
 pub type ClientId = u16;
@@ -74,6 +78,8 @@ pub enum ServerInstruction {
         Box<Layout>,
         Box<PluginAliases>,
         bool, // should launch setup wizard
+        bool, // is_web_client
+        bool, // layout_is_welcome_screen
         ClientId,
     ),
     Render(Option<HashMap<ClientId, String>>),
@@ -89,6 +95,7 @@ pub enum ServerInstruction {
         Options,             // represents the runtime configuration options
         Option<usize>,       // tab position to focus
         Option<(u32, bool)>, // (pane_id, is_plugin) => pane_id to focus
+        bool,                // is_web_client
         ClientId,
     ),
     ConnStatus(ClientId),
@@ -117,6 +124,12 @@ pub enum ServerInstruction {
         keys_to_unbind: Vec<(InputMode, KeyWithModifier)>,
         write_config_to_disk: bool,
     },
+    StartWebServer(ClientId),
+    ShareCurrentSession(ClientId),
+    StopSharingCurrentSession(ClientId),
+    SendWebClientsForbidden(ClientId),
+    WebServerStarted(String), // String -> base_url
+    FailedToStartWebServer(String),
 }
 
 impl From<&ServerInstruction> for ServerContext {
@@ -153,6 +166,16 @@ impl From<&ServerInstruction> for ServerContext {
                 ServerContext::FailedToWriteConfigToDisk
             },
             ServerInstruction::RebindKeys { .. } => ServerContext::RebindKeys,
+            ServerInstruction::StartWebServer(..) => ServerContext::StartWebServer,
+            ServerInstruction::ShareCurrentSession(..) => ServerContext::ShareCurrentSession,
+            ServerInstruction::StopSharingCurrentSession(..) => {
+                ServerContext::StopSharingCurrentSession
+            },
+            ServerInstruction::WebServerStarted(..) => ServerContext::WebServerStarted,
+            ServerInstruction::FailedToStartWebServer(..) => ServerContext::FailedToStartWebServer,
+            ServerInstruction::SendWebClientsForbidden(..) => {
+                ServerContext::SendWebClientsForbidden
+            },
         }
     }
 }
@@ -302,7 +325,10 @@ pub(crate) struct SessionMetaData {
     pub layout: Box<Layout>,
     pub current_input_modes: HashMap<ClientId, InputMode>,
     pub session_configuration: SessionConfiguration,
-
+    pub web_sharing: WebSharing, // this is a special attribute explicitly set on session
+    // initialization because we don't want it to be overridden by
+    // configuration changes, the only way it can be overwritten is by
+    // explicit plugin action
     screen_thread: Option<thread::JoinHandle<()>>,
     pty_thread: Option<thread::JoinHandle<()>>,
     plugin_thread: Option<thread::JoinHandle<()>>,
@@ -344,6 +370,7 @@ impl SessionMetaData {
                 TerminalAction::RunCommand(RunCommand {
                     command: shell.clone(),
                     cwd: new_config.options.default_cwd.clone(),
+                    use_terminal_title: true,
                     ..Default::default()
                 })
             });
@@ -369,6 +396,10 @@ impl SessionMetaData {
                     hide_session_name: new_config.ui.pane_frames.hide_session_name,
                     stacked_resize: new_config.options.stacked_resize.unwrap_or(true),
                     default_editor: new_config.options.scrollback_editor.clone(),
+                    advanced_mouse_actions: new_config
+                        .options
+                        .advanced_mouse_actions
+                        .unwrap_or(true),
                 })
                 .unwrap();
             self.senders
@@ -384,6 +415,7 @@ impl SessionMetaData {
                 .send_to_pty(PtyInstruction::Reconfigure {
                     client_id,
                     default_editor: new_config.options.scrollback_editor,
+                    post_command_discovery_hook: new_config.options.post_command_discovery_hook,
                 })
                 .unwrap();
         }
@@ -448,8 +480,8 @@ macro_rules! send_to_client {
 
 #[derive(Clone, Debug, PartialEq)]
 pub(crate) struct SessionState {
-    clients: HashMap<ClientId, Option<Size>>,
-    pipes: HashMap<String, ClientId>, // String => pipe_id
+    clients: HashMap<ClientId, Option<(Size, bool)>>, // bool -> is_web_client
+    pipes: HashMap<String, ClientId>,                 // String => pipe_id
 }
 
 impl SessionState {
@@ -480,20 +512,31 @@ impl SessionState {
         self.pipes.retain(|_p_id, c_id| c_id != &client_id);
     }
     pub fn set_client_size(&mut self, client_id: ClientId, size: Size) {
-        self.clients.insert(client_id, Some(size));
+        self.clients
+            .entry(client_id)
+            .or_insert_with(Default::default)
+            .as_mut()
+            .map(|(s, _is_web_client)| *s = size);
+    }
+    pub fn set_client_data(&mut self, client_id: ClientId, size: Size, is_web_client: bool) {
+        self.clients.insert(client_id, Some((size, is_web_client)));
     }
     pub fn min_client_terminal_size(&self) -> Option<Size> {
         // None if there are no client sizes
         let mut rows: Vec<usize> = self
             .clients
             .values()
-            .filter_map(|size| size.map(|size| size.rows))
+            .filter_map(|size_and_is_web_client| {
+                size_and_is_web_client.map(|(size, _is_web_client)| size.rows)
+            })
             .collect();
         rows.sort_unstable();
         let mut cols: Vec<usize> = self
             .clients
             .values()
-            .filter_map(|size| size.map(|size| size.cols))
+            .filter_map(|size_and_is_web_client| {
+                size_and_is_web_client.map(|(size, _is_web_client)| size.cols)
+            })
             .collect();
         cols.sort_unstable();
         let min_rows = rows.first();
@@ -509,6 +552,15 @@ impl SessionState {
     pub fn client_ids(&self) -> Vec<ClientId> {
         self.clients.keys().copied().collect()
     }
+    pub fn web_client_ids(&self) -> Vec<ClientId> {
+        self.clients
+            .iter()
+            .filter_map(|(c_id, size_and_is_web_client)| {
+                size_and_is_web_client
+                    .and_then(|(_s, is_web_client)| if is_web_client { Some(*c_id) } else { None })
+            })
+            .collect()
+    }
     pub fn get_pipe(&self, pipe_name: &str) -> Option<ClientId> {
         self.pipes.get(pipe_name).copied()
     }
@@ -555,9 +607,8 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
     let _ = thread::Builder::new()
         .name("server_listener".to_string())
         .spawn({
-            use zellij_utils::{
-                interprocess::local_socket::LocalSocketListener, shared::set_permissions,
-            };
+            use interprocess::local_socket::LocalSocketListener;
+            use zellij_utils::shared::set_permissions;
 
             let os_input = os_input.clone();
             let session_data = session_data.clone();
@@ -617,6 +668,8 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                 layout,
                 plugin_aliases,
                 should_launch_setup_wizard,
+                is_web_client,
+                layout_is_welcome_screen,
                 client_id,
             ) => {
                 let mut session = init_session(
@@ -646,15 +699,17 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                     .insert(client_id, default_input_mode);
 
                 *session_data.write().unwrap() = Some(session);
-                session_state
-                    .write()
-                    .unwrap()
-                    .set_client_size(client_id, client_attributes.size);
+                session_state.write().unwrap().set_client_data(
+                    client_id,
+                    client_attributes.size,
+                    is_web_client,
+                );
 
                 let default_shell = runtime_config_options.default_shell.map(|shell| {
                     TerminalAction::RunCommand(RunCommand {
                         command: shell,
                         cwd: config.options.default_cwd.clone(),
+                        use_terminal_title: true,
                         ..Default::default()
                     })
                 });
@@ -679,7 +734,7 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                             tab_name,
                             swap_layouts,
                             should_focus_tab,
-                            client_id,
+                            (client_id, is_web_client),
                         ))
                         .unwrap()
                 };
@@ -709,10 +764,16 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                         // intrusive
                         let setup_wizard = setup_wizard_floating_pane();
                         floating_panes.push(setup_wizard);
-                    } else if should_show_release_notes(runtime_config_options.show_release_notes) {
+                    } else if should_show_release_notes(
+                        runtime_config_options.show_release_notes,
+                        layout_is_welcome_screen,
+                    ) {
                         let about = about_floating_pane();
                         floating_panes.push(about);
-                    } else if should_show_startup_tip(runtime_config_options.show_startup_tips) {
+                    } else if should_show_startup_tip(
+                        runtime_config_options.show_startup_tips,
+                        layout_is_welcome_screen,
+                    ) {
                         let tip = tip_floating_pane();
                         floating_panes.push(tip);
                     }
@@ -742,6 +803,7 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                 runtime_config_options,
                 tab_position_to_focus,
                 pane_id_to_focus,
+                is_web_client,
                 client_id,
             ) => {
                 let mut rlock = session_data.write().unwrap();
@@ -761,10 +823,11 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                     .current_input_modes
                     .insert(client_id, default_input_mode);
 
-                session_state
-                    .write()
-                    .unwrap()
-                    .set_client_size(client_id, attrs.size);
+                session_state.write().unwrap().set_client_data(
+                    client_id,
+                    attrs.size,
+                    is_web_client,
+                );
                 let min_size = session_state
                     .read()
                     .unwrap()
@@ -778,6 +841,7 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                     .senders
                     .send_to_screen(ScreenInstruction::AddClient(
                         client_id,
+                        is_web_client,
                         tab_position_to_focus,
                         pane_id_to_focus,
                     ))
@@ -945,6 +1009,23 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                     .send_to_plugin(PluginInstruction::RemoveClient(client_id))
                     .unwrap();
             },
+            ServerInstruction::SendWebClientsForbidden(client_id) => {
+                let _ = os_input.send_to_client(
+                    client_id,
+                    ServerToClientMsg::Exit(ExitReason::WebClientsForbidden),
+                );
+                remove_client!(client_id, os_input, session_state);
+                if let Some(min_size) = session_state.read().unwrap().min_client_terminal_size() {
+                    session_data
+                        .write()
+                        .unwrap()
+                        .as_ref()
+                        .unwrap()
+                        .senders
+                        .send_to_screen(ScreenInstruction::TerminalResize(min_size))
+                        .unwrap();
+                }
+            },
             ServerInstruction::KillSession => {
                 let client_ids = session_state.read().unwrap().client_ids();
                 for client_id in client_ids {
@@ -1242,6 +1323,98 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                     }
                 }
             },
+            ServerInstruction::StartWebServer(client_id) => {
+                if cfg!(feature = "web_server_capability") {
+                    send_to_client!(
+                        client_id,
+                        os_input,
+                        ServerToClientMsg::StartWebServer,
+                        session_state
+                    );
+                } else {
+                    // TODO: test this
+                    log::error!("Cannot start web server: this instance of Zellij was compiled without web_server_capability");
+                }
+            },
+            ServerInstruction::ShareCurrentSession(_client_id) => {
+                if cfg!(feature = "web_server_capability") {
+                    let successfully_changed = session_data
+                        .write()
+                        .ok()
+                        .and_then(|mut s| s.as_mut().map(|s| s.web_sharing.set_sharing()))
+                        .unwrap_or(false);
+                    if successfully_changed {
+                        session_data
+                            .write()
+                            .unwrap()
+                            .as_ref()
+                            .unwrap()
+                            .senders
+                            .send_to_screen(ScreenInstruction::SessionSharingStatusChange(true))
+                            .unwrap();
+                    }
+                } else {
+                    log::error!("Cannot share session: this instance of Zellij was compiled without web_server_capability");
+                }
+            },
+            ServerInstruction::StopSharingCurrentSession(_client_id) => {
+                if cfg!(feature = "web_server_capability") {
+                    let successfully_changed = session_data
+                        .write()
+                        .ok()
+                        .and_then(|mut s| s.as_mut().map(|s| s.web_sharing.set_not_sharing()))
+                        .unwrap_or(false);
+                    if successfully_changed {
+                        // disconnect existing web clients
+                        let web_client_ids: Vec<ClientId> = session_state
+                            .read()
+                            .unwrap()
+                            .web_client_ids()
+                            .iter()
+                            .copied()
+                            .collect();
+                        for client_id in web_client_ids {
+                            let _ = os_input.send_to_client(
+                                client_id,
+                                ServerToClientMsg::Exit(ExitReason::WebClientsForbidden),
+                            );
+                            remove_client!(client_id, os_input, session_state);
+                        }
+
+                        session_data
+                            .write()
+                            .unwrap()
+                            .as_ref()
+                            .unwrap()
+                            .senders
+                            .send_to_screen(ScreenInstruction::SessionSharingStatusChange(false))
+                            .unwrap();
+                    }
+                } else {
+                    // TODO: test this
+                    log::error!("Cannot start web server: this instance of Zellij was compiled without web_server_capability");
+                }
+            },
+            ServerInstruction::WebServerStarted(base_url) => {
+                session_data
+                    .write()
+                    .unwrap()
+                    .as_ref()
+                    .unwrap()
+                    .senders
+                    .send_to_plugin(PluginInstruction::WebServerStarted(base_url))
+                    .unwrap();
+            },
+            ServerInstruction::FailedToStartWebServer(error) => {
+                session_data
+                    .write()
+                    .unwrap()
+                    .as_ref()
+                    .unwrap()
+                    .senders
+                    .send_to_plugin(PluginInstruction::FailedToStartWebServer(error))
+                    .unwrap();
+            },
         }
     }
 
@@ -1308,10 +1481,18 @@ fn init_session(
 
     let serialization_interval = config_options.serialization_interval;
     let disable_session_metadata = config_options.disable_session_metadata.unwrap_or(false);
+    let web_server_ip = config_options
+        .web_server_ip
+        .unwrap_or_else(|| IpAddr::V4(Ipv4Addr::new(127, 0, 0, 1)));
+    let web_server_port = config_options.web_server_port.unwrap_or_else(|| 8082);
+    let has_certificate =
+        config_options.web_server_cert.is_some() && config_options.web_server_key.is_some();
+    let enforce_https_for_localhost = config_options.enforce_https_for_localhost.unwrap_or(false);
 
     let default_shell = config_options.default_shell.clone().map(|command| {
         TerminalAction::RunCommand(RunCommand {
             command,
+            use_terminal_title: true,
             ..Default::default()
         })
     });
@@ -1340,6 +1521,7 @@ fn init_session(
                 ),
                 opts.debug,
                 config_options.scrollback_editor.clone(),
+                config_options.post_command_discovery_hook.clone(),
             );
 
             move || pty_thread_main(pty, layout.clone()).fatal()
@@ -1455,11 +1637,18 @@ fn init_session(
                 None,
                 Some(os_input.clone()),
             );
+            let web_server_base_url = web_server_base_url(
+                web_server_ip,
+                web_server_port,
+                has_certificate,
+                enforce_https_for_localhost,
+            );
             move || {
                 background_jobs_main(
                     background_jobs_bus,
                     serialization_interval,
                     disable_session_metadata,
+                    web_server_base_url,
                 )
                 .fatal()
             }
@@ -1487,6 +1676,10 @@ fn init_session(
         plugin_thread: Some(plugin_thread),
         pty_writer_thread: Some(pty_writer_thread),
         background_jobs_thread: Some(background_jobs_thread),
+        #[cfg(feature = "web_server_capability")]
+        web_sharing: config.options.web_sharing.unwrap_or(WebSharing::Off),
+        #[cfg(not(feature = "web_server_capability"))]
+        web_sharing: WebSharing::Disabled,
     }
 }
 
@@ -1523,7 +1716,13 @@ fn tip_floating_pane() -> FloatingPaneLayout {
     about_pane
 }
 
-fn should_show_release_notes(should_show_release_notes_config: Option<bool>) -> bool {
+fn should_show_release_notes(
+    should_show_release_notes_config: Option<bool>,
+    layout_is_welcome_screen: bool,
+) -> bool {
+    if layout_is_welcome_screen {
+        return false;
+    }
     if let Some(should_show_release_notes_config) = should_show_release_notes_config {
         if !should_show_release_notes_config {
             // if we were explicitly told not to show release notes, we don't show them,
@@ -1546,8 +1745,15 @@ fn should_show_release_notes(should_show_release_notes_config: Option<bool>) ->
     }
 }
 
-fn should_show_startup_tip(should_show_startup_tip_config: Option<bool>) -> bool {
-    should_show_startup_tip_config.unwrap_or(true)
+fn should_show_startup_tip(
+    should_show_startup_tip_config: Option<bool>,
+    layout_is_welcome_screen: bool,
+) -> bool {
+    if layout_is_welcome_screen {
+        false
+    } else {
+        should_show_startup_tip_config.unwrap_or(true)
+    }
 }
 
 #[cfg(not(feature = "singlepass"))]
