@@ -1,16 +1,15 @@
 use super::{is_too_high, is_too_wide, stringify_text, Coordinates, Text};
-use crate::panes::terminal_character::{AnsiCode, RESET_STYLES};
-use std::collections::BTreeMap;
-use zellij_utils::{
-    data::{PaletteColor, Style},
-    shared::ansi_len,
+use crate::panes::{
+    terminal_character::{AnsiCode, RESET_STYLES},
+    CharacterStyles,
 };
+use std::collections::BTreeMap;
+use zellij_utils::{data::Style, shared::ansi_len};
 
 pub fn table(
     columns: usize,
     _rows: usize,
     contents: Vec<Text>,
-    title_color: Option<PaletteColor>,
     style: &Style,
     coordinates: Option<Coordinates>,
 ) -> Vec<u8> {
@@ -18,34 +17,37 @@ pub fn table(
     // we first arrange the data by columns so that we can pad them by the widest one
     let stringified_columns = stringify_table_columns(contents, columns);
     let stringified_rows = stringify_table_rows(stringified_columns, &coordinates);
-    let title_styles = RESET_STYLES
-        .foreground(title_color.map(|t| t.into()))
-        .bold(Some(AnsiCode::On));
-    let cell_styles = RESET_STYLES.bold(Some(AnsiCode::On));
     for (row_index, (_, row)) in stringified_rows.into_iter().enumerate() {
         let is_title_row = row_index == 0;
         if is_too_high(row_index + 1, &coordinates) {
             break;
         }
         for cell in row {
-            let mut reset_styles_for_item = RESET_STYLES;
-            let mut text_style = if is_title_row {
-                title_styles
+            let declaration = if is_title_row {
+                style.colors.table_title
             } else {
-                cell_styles
+                if cell.selected {
+                    style.colors.table_cell_selected
+                } else {
+                    style.colors.table_cell_unselected
+                }
+            };
+
+            let text_style = if cell.opaque || cell.selected {
+                CharacterStyles::from(declaration).background(Some(declaration.background.into()))
+            } else {
+                CharacterStyles::from(declaration)
             };
-            if cell.selected {
-                reset_styles_for_item.background = None;
-                text_style = text_style.background(Some(style.colors.bg.into()));
-            } else if cell.opaque {
-                reset_styles_for_item.background = None;
-                text_style = text_style.background(Some(style.colors.black.into()));
-            }
             // here we intentionally don't pass our coordinates even if we have them, because
             // these cells have already been padded and truncated
-            let (text, _text_width) =
-                stringify_text(&cell, None, &None, style, text_style, cell.selected);
-            stringified.push_str(&format!("{}{}{} ", text_style, text, reset_styles_for_item));
+            let (text, _text_width) = stringify_text(
+                &cell,
+                None,
+                &None,
+                &declaration,
+                text_style.bold(Some(AnsiCode::On)),
+            );
+            stringified.push_str(&format!("{}{} {}", text_style, text, RESET_STYLES));
         }
         let next_row_instruction = coordinates
             .as_ref()
