@@ -4,8 +4,9 @@ use crate::{
     consts::{ZELLIJ_CONFIG_DIR_ENV, ZELLIJ_CONFIG_FILE_ENV},
     input::{layout::PluginUserConfiguration, options::CliOptions},
 };
-use clap::{Parser, Subcommand};
+use clap::{Args, Parser, Subcommand};
 use serde::{Deserialize, Serialize};
+use std::net::IpAddr;
 use std::path::PathBuf;
 use url::Url;
 
@@ -89,11 +90,103 @@ pub enum Command {
     #[clap(name = "setup", value_parser)]
     Setup(Setup),
 
+    /// Run a web server to serve terminal sessions
+    #[clap(name = "web", value_parser)]
+    Web(WebCli),
+
     /// Explore existing zellij sessions
     #[clap(flatten)]
     Sessions(Sessions),
 }
 
+#[derive(Debug, Clone, Args, Serialize, Deserialize)]
+pub struct WebCli {
+    /// Start the server (default unless other arguments are specified)
+    #[clap(long, value_parser, display_order = 1)]
+    pub start: bool,
+
+    /// Stop the server
+    #[clap(long, value_parser, exclusive(true), display_order = 2)]
+    pub stop: bool,
+
+    /// Get the server status
+    #[clap(long, value_parser, exclusive(true), display_order = 3)]
+    pub status: bool,
+
+    /// Run the server in the background
+    #[clap(
+        short,
+        long,
+        value_parser,
+        conflicts_with_all(&["stop", "status", "create-token", "revoke-token", "revoke-all-tokens"]),
+        display_order = 4
+    )]
+    pub daemonize: bool,
+    /// Create a login token for the web interface, will only be displayed once and cannot later be
+    /// retrieved. Returns the token name and the token.
+    #[clap(long, value_parser, exclusive(true), display_order = 5)]
+    pub create_token: bool,
+    /// Revoke a login token by its name
+    #[clap(
+        long,
+        value_parser,
+        exclusive(true),
+        value_name = "TOKEN NAME",
+        display_order = 6
+    )]
+    pub revoke_token: Option<String>,
+    /// Revoke all login tokens
+    #[clap(long, value_parser, exclusive(true), display_order = 7)]
+    pub revoke_all_tokens: bool,
+    /// List token names and their creation dates (cannot show actual tokens)
+    #[clap(long, value_parser, exclusive(true), display_order = 8)]
+    pub list_tokens: bool,
+    /// The ip address to listen on locally for connections (defaults to 127.0.0.1)
+    #[clap(
+        long,
+        value_parser,
+        conflicts_with_all(&["stop", "status", "create-token", "revoke-token", "revoke-all-tokens"]),
+        display_order = 9
+    )]
+    pub ip: Option<IpAddr>,
+    /// The port to listen on locally for connections (defaults to 8082)
+    #[clap(
+        long,
+        value_parser,
+        conflicts_with_all(&["stop", "status", "create-token", "revoke-token", "revoke-all-tokens"]),
+        display_order = 10
+    )]
+    pub port: Option<u16>,
+    /// The path to the SSL certificate (required if not listening on 127.0.0.1)
+    #[clap(
+        long,
+        value_parser,
+        conflicts_with_all(&["stop", "status", "create-token", "revoke-token", "revoke-all-tokens"]),
+        display_order = 11
+    )]
+    pub cert: Option<PathBuf>,
+    /// The path to the SSL key (required if not listening on 127.0.0.1)
+    #[clap(
+        long,
+        value_parser,
+        conflicts_with_all(&["stop", "status", "create-token", "revoke-token", "revoke-all-tokens"]),
+        display_order = 12
+    )]
+    pub key: Option<PathBuf>,
+}
+
+impl WebCli {
+    pub fn get_start(&self) -> bool {
+        self.start
+            || !(self.stop
+                || self.status
+                || self.create_token
+                || self.revoke_token.is_some()
+                || self.revoke_all_tokens
+                || self.list_tokens)
+    }
+}
+
 #[derive(Debug, Subcommand, Clone, Serialize, Deserialize)]
 pub enum SessionCommand {
     /// Change the behaviour of zellij
@@ -246,6 +339,18 @@ pub enum Sessions {
         /// The height if the pane is floating as a bare integer (eg. 1) or percent (eg. 10%)
         #[clap(long, requires("floating"))]
         height: Option<String>,
+        /// Whether to pin a floating pane so that it is always on top
+        #[clap(long, requires("floating"))]
+        pinned: Option<bool>,
+        #[clap(
+            long,
+            conflicts_with("floating"),
+            conflicts_with("direction"),
+            value_parser,
+            default_value("false"),
+            takes_value(false)
+        )]
+        stacked: bool,
     },
     /// Load a plugin
     #[clap(visible_alias = "p")]
@@ -288,6 +393,9 @@ pub enum Sessions {
         /// The height if the pane is floating as a bare integer (eg. 1) or percent (eg. 10%)
         #[clap(long, requires("floating"))]
         height: Option<String>,
+        /// Whether to pin a floating pane so that it is always on top
+        #[clap(long, requires("floating"))]
+        pinned: Option<bool>,
     },
     /// Edit file with default $EDITOR / $VISUAL
     #[clap(visible_alias = "e")]
@@ -333,6 +441,9 @@ pub enum Sessions {
         /// The height if the pane is floating as a bare integer (eg. 1) or percent (eg. 10%)
         #[clap(long, requires("floating"))]
         height: Option<String>,
+        /// Whether to pin a floating pane so that it is always on top
+        #[clap(long, requires("floating"))]
+        pinned: Option<bool>,
     },
     ConvertConfig {
         old_config_file: PathBuf,
@@ -526,6 +637,18 @@ pub enum CliAction {
         /// The height if the pane is floating as a bare integer (eg. 1) or percent (eg. 10%)
         #[clap(long, requires("floating"))]
         height: Option<String>,
+        /// Whether to pin a floating pane so that it is always on top
+        #[clap(long, requires("floating"))]
+        pinned: Option<bool>,
+        #[clap(
+            long,
+            conflicts_with("floating"),
+            conflicts_with("direction"),
+            value_parser,
+            default_value("false"),
+            takes_value(false)
+        )]
+        stacked: bool,
     },
     /// Open the specified file in a new zellij pane with your default EDITOR
     Edit {
@@ -570,6 +693,9 @@ pub enum CliAction {
         /// The height if the pane is floating as a bare integer (eg. 1) or percent (eg. 10%)
         #[clap(long, requires("floating"))]
         height: Option<String>,
+        /// Whether to pin a floating pane so that it is always on top
+        #[clap(long, requires("floating"))]
+        pinned: Option<bool>,
     },
     /// Switch input mode of all connected clients [locked|pane|tab|resize|move|search|session]
     SwitchMode {
@@ -625,7 +751,7 @@ pub enum CliAction {
         name: Option<String>,
 
         /// Change the working directory of the new tab
-        #[clap(short, long, value_parser, requires("layout"))]
+        #[clap(short, long, value_parser)]
         cwd: Option<PathBuf>,
     },
     /// Move the focused tab in the specified direction. [right|left]
@@ -744,4 +870,37 @@ tail -f /tmp/my-live-logfile | zellij action pipe --name logs --plugin https://e
         plugin_title: Option<String>,
     },
     ListClients,
+    TogglePanePinned,
+    /// Stack pane ids
+    /// Ids are a space separated list of pane ids.
+    /// They should either be in the form of `terminal_<int>` (eg. terminal_1), `plugin_<int>` (eg.
+    /// plugin_1) or bare integers in which case they'll be considered terminals (eg. 1 is
+    /// the equivalent of terminal_1)
+    ///
+    /// Example: zellij action stack-panes -- terminal_1 plugin_2 3
+    StackPanes {
+        #[clap(last(true), required(true))]
+        pane_ids: Vec<String>,
+    },
+    ChangeFloatingPaneCoordinates {
+        /// The pane_id of the floating pane, eg.  terminal_1, plugin_2 or 3 (equivalent to
+        /// terminal_3)
+        #[clap(short, long, value_parser)]
+        pane_id: String,
+        /// The x coordinates if the pane is floating as a bare integer (eg. 1) or percent (eg. 10%)
+        #[clap(short, long)]
+        x: Option<String>,
+        /// The y coordinates if the pane is floating as a bare integer (eg. 1) or percent (eg. 10%)
+        #[clap(short, long)]
+        y: Option<String>,
+        /// The width if the pane is floating as a bare integer (eg. 1) or percent (eg. 10%)
+        #[clap(long)]
+        width: Option<String>,
+        /// The height if the pane is floating as a bare integer (eg. 1) or percent (eg. 10%)
+        #[clap(long)]
+        height: Option<String>,
+        /// Whether to pin a floating pane so that it is always on top
+        #[clap(long)]
+        pinned: Option<bool>,
+    },
 }
