@@ -64,6 +64,9 @@ impl PluginConfig {
                     || tag == "session-manager"
                     || tag == "configuration"
                     || tag == "plugin-manager"
+                    || tag == "about"
+                    || tag == "share"
+                    || tag == "multiple-select"
                 {
                     Some(PluginConfig {
                         path: PathBuf::from(&tag),
