@@ -2,7 +2,8 @@ pub use super::generated_api::api::{
     action::{
         action::OptionalPayload, Action as ProtobufAction, ActionName as ProtobufActionName,
         DumpScreenPayload, EditFilePayload, GoToTabNamePayload, IdAndName,
-        LaunchOrFocusPluginPayload, MovePanePayload, MoveTabDirection as ProtobufMoveTabDirection,
+        LaunchOrFocusPluginPayload, MouseEventPayload as ProtobufMouseEventPayload,
+        MovePanePayload, MoveTabDirection as ProtobufMoveTabDirection,
         NameAndValue as ProtobufNameAndValue, NewFloatingPanePayload, NewPanePayload,
         NewPluginPanePayload, NewTiledPanePayload, PaneIdAndShouldFloat,
         PluginConfiguration as ProtobufPluginConfiguration, Position as ProtobufPosition,
@@ -21,6 +22,7 @@ use crate::input::command::{OpenFilePayload, RunCommandAction};
 use crate::input::layout::{
     PluginUserConfiguration, RunPlugin, RunPluginLocation, RunPluginOrAlias,
 };
+use crate::input::mouse::{MouseEvent, MouseEventType};
 use crate::position::Position;
 
 use std::collections::BTreeMap;
@@ -313,7 +315,7 @@ impl TryFrom<ProtobufAction> for Action {
                     Some(_) => Err("NewTab should not have a payload"),
                     None => {
                         // we do not serialize the layouts of this action
-                        Ok(Action::NewTab(None, vec![], None, None, None, true))
+                        Ok(Action::NewTab(None, vec![], None, None, None, true, None))
                     },
                 }
             },
@@ -382,21 +384,27 @@ impl TryFrom<ProtobufAction> for Action {
             Some(ProtobufActionName::LeftClick) => match protobuf_action.optional_payload {
                 Some(OptionalPayload::LeftClickPayload(payload)) => {
                     let position = payload.try_into()?;
-                    Ok(Action::LeftClick(position))
+                    Ok(Action::MouseEvent(MouseEvent::new_left_press_event(
+                        position,
+                    )))
                 },
                 _ => Err("Wrong payload for Action::LeftClick"),
             },
             Some(ProtobufActionName::RightClick) => match protobuf_action.optional_payload {
                 Some(OptionalPayload::RightClickPayload(payload)) => {
                     let position = payload.try_into()?;
-                    Ok(Action::RightClick(position))
+                    Ok(Action::MouseEvent(MouseEvent::new_right_press_event(
+                        position,
+                    )))
                 },
                 _ => Err("Wrong payload for Action::RightClick"),
             },
             Some(ProtobufActionName::MiddleClick) => match protobuf_action.optional_payload {
                 Some(OptionalPayload::MiddleClickPayload(payload)) => {
                     let position = payload.try_into()?;
-                    Ok(Action::MiddleClick(position))
+                    Ok(Action::MouseEvent(MouseEvent::new_middle_press_event(
+                        position,
+                    )))
                 },
                 _ => Err("Wrong payload for Action::MiddleClick"),
             },
@@ -460,14 +468,18 @@ impl TryFrom<ProtobufAction> for Action {
             Some(ProtobufActionName::LeftMouseRelease) => match protobuf_action.optional_payload {
                 Some(OptionalPayload::LeftMouseReleasePayload(payload)) => {
                     let position = payload.try_into()?;
-                    Ok(Action::LeftMouseRelease(position))
+                    Ok(Action::MouseEvent(MouseEvent::new_left_release_event(
+                        position,
+                    )))
                 },
                 _ => Err("Wrong payload for Action::LeftMouseRelease"),
             },
             Some(ProtobufActionName::RightMouseRelease) => match protobuf_action.optional_payload {
                 Some(OptionalPayload::RightMouseReleasePayload(payload)) => {
                     let position = payload.try_into()?;
-                    Ok(Action::RightMouseRelease(position))
+                    Ok(Action::MouseEvent(MouseEvent::new_right_release_event(
+                        position,
+                    )))
                 },
                 _ => Err("Wrong payload for Action::RightMouseRelease"),
             },
@@ -475,31 +487,19 @@ impl TryFrom<ProtobufAction> for Action {
                 match protobuf_action.optional_payload {
                     Some(OptionalPayload::MiddleMouseReleasePayload(payload)) => {
                         let position = payload.try_into()?;
-                        Ok(Action::MiddleMouseRelease(position))
+                        Ok(Action::MouseEvent(MouseEvent::new_middle_release_event(
+                            position,
+                        )))
                     },
                     _ => Err("Wrong payload for Action::MiddleMouseRelease"),
                 }
             },
-            Some(ProtobufActionName::MouseHoldLeft) => match protobuf_action.optional_payload {
-                Some(OptionalPayload::MouseHoldLeftPayload(payload)) => {
-                    let position = payload.try_into()?;
-                    Ok(Action::MouseHoldLeft(position))
+            Some(ProtobufActionName::MouseEvent) => match protobuf_action.optional_payload {
+                Some(OptionalPayload::MouseEventPayload(payload)) => {
+                    let event = payload.try_into()?;
+                    Ok(Action::MouseEvent(event))
                 },
-                _ => Err("Wrong payload for Action::MouseHoldLeft"),
-            },
-            Some(ProtobufActionName::MouseHoldRight) => match protobuf_action.optional_payload {
-                Some(OptionalPayload::MouseHoldRightPayload(payload)) => {
-                    let position = payload.try_into()?;
-                    Ok(Action::MouseHoldRight(position))
-                },
-                _ => Err("Wrong payload for Action::MouseHoldRight"),
-            },
-            Some(ProtobufActionName::MouseHoldMiddle) => match protobuf_action.optional_payload {
-                Some(OptionalPayload::MouseHoldMiddlePayload(payload)) => {
-                    let position = payload.try_into()?;
-                    Ok(Action::MouseHoldMiddle(position))
-                },
-                _ => Err("Wrong payload for Action::MouseHoldMiddle"),
+                _ => Err("Wrong payload for Action::MouseEvent"),
             },
             Some(ProtobufActionName::SearchInput) => match protobuf_action.optional_payload {
                 Some(OptionalPayload::SearchInputPayload(payload)) => {
@@ -688,6 +688,20 @@ impl TryFrom<ProtobufAction> for Action {
                 },
                 _ => Err("Wrong payload for Action::RenameSession"),
             },
+            Some(ProtobufActionName::TogglePanePinned) => match protobuf_action.optional_payload {
+                Some(_) => Err("TogglePanePinned should not have a payload"),
+                None => Ok(Action::TogglePanePinned),
+            },
+            Some(ProtobufActionName::TogglePaneInGroup) => match protobuf_action.optional_payload {
+                Some(_) => Err("TogglePaneInGroup should not have a payload"),
+                None => Ok(Action::TogglePaneInGroup),
+            },
+            Some(ProtobufActionName::ToggleGroupMarking) => {
+                match protobuf_action.optional_payload {
+                    Some(_) => Err("ToggleGroupMarking should not have a payload"),
+                    None => Ok(Action::ToggleGroupMarking),
+                }
+            },
             Some(ProtobufActionName::KeybindPipe) => match protobuf_action.optional_payload {
                 Some(_) => Err("KeybindPipe should not have a payload"),
                 // TODO: at some point we might want to support a payload here
@@ -706,6 +720,10 @@ impl TryFrom<ProtobufAction> for Action {
                     plugin_id: None,
                 }),
             },
+            Some(ProtobufActionName::NewStackedPane) => match protobuf_action.optional_payload {
+                Some(_) => Err("NewStackedPane should not have a payload"),
+                None => Ok(Action::NewStackedPane(None, None)),
+            },
             _ => Err("Unknown Action"),
         }
     }
@@ -1036,27 +1054,6 @@ impl TryFrom<Action> for ProtobufAction {
                 name: ProtobufActionName::Detach as i32,
                 optional_payload: None,
             }),
-            Action::LeftClick(position) => {
-                let position: ProtobufPosition = position.try_into()?;
-                Ok(ProtobufAction {
-                    name: ProtobufActionName::LeftClick as i32,
-                    optional_payload: Some(OptionalPayload::LeftClickPayload(position)),
-                })
-            },
-            Action::RightClick(position) => {
-                let position: ProtobufPosition = position.try_into()?;
-                Ok(ProtobufAction {
-                    name: ProtobufActionName::RightClick as i32,
-                    optional_payload: Some(OptionalPayload::RightClickPayload(position)),
-                })
-            },
-            Action::MiddleClick(position) => {
-                let position: ProtobufPosition = position.try_into()?;
-                Ok(ProtobufAction {
-                    name: ProtobufActionName::MiddleClick as i32,
-                    optional_payload: Some(OptionalPayload::MiddleClickPayload(position)),
-                })
-            },
             Action::LaunchOrFocusPlugin(
                 run_plugin_or_alias,
                 should_float,
@@ -1101,46 +1098,11 @@ impl TryFrom<Action> for ProtobufAction {
                     )),
                 })
             },
-            Action::LeftMouseRelease(position) => {
-                let position: ProtobufPosition = position.try_into()?;
+            Action::MouseEvent(event) => {
+                let payload: ProtobufMouseEventPayload = event.try_into()?;
                 Ok(ProtobufAction {
-                    name: ProtobufActionName::LeftMouseRelease as i32,
-                    optional_payload: Some(OptionalPayload::LeftMouseReleasePayload(position)),
-                })
-            },
-            Action::RightMouseRelease(position) => {
-                let position: ProtobufPosition = position.try_into()?;
-                Ok(ProtobufAction {
-                    name: ProtobufActionName::RightMouseRelease as i32,
-                    optional_payload: Some(OptionalPayload::RightMouseReleasePayload(position)),
-                })
-            },
-            Action::MiddleMouseRelease(position) => {
-                let position: ProtobufPosition = position.try_into()?;
-                Ok(ProtobufAction {
-                    name: ProtobufActionName::MiddleMouseRelease as i32,
-                    optional_payload: Some(OptionalPayload::MiddleMouseReleasePayload(position)),
-                })
-            },
-            Action::MouseHoldLeft(position) => {
-                let position: ProtobufPosition = position.try_into()?;
-                Ok(ProtobufAction {
-                    name: ProtobufActionName::MouseHoldLeft as i32,
-                    optional_payload: Some(OptionalPayload::MouseHoldLeftPayload(position)),
-                })
-            },
-            Action::MouseHoldRight(position) => {
-                let position: ProtobufPosition = position.try_into()?;
-                Ok(ProtobufAction {
-                    name: ProtobufActionName::MouseHoldRight as i32,
-                    optional_payload: Some(OptionalPayload::MouseHoldRightPayload(position)),
-                })
-            },
-            Action::MouseHoldMiddle(position) => {
-                let position: ProtobufPosition = position.try_into()?;
-                Ok(ProtobufAction {
-                    name: ProtobufActionName::MouseHoldMiddle as i32,
-                    optional_payload: Some(OptionalPayload::MouseHoldMiddlePayload(position)),
+                    name: ProtobufActionName::MouseEvent as i32,
+                    optional_payload: Some(OptionalPayload::MouseEventPayload(payload)),
                 })
             },
             Action::SearchInput(bytes) => Ok(ProtobufAction {
@@ -1284,6 +1246,22 @@ impl TryFrom<Action> for ProtobufAction {
                 name: ProtobufActionName::KeybindPipe as i32,
                 optional_payload: None,
             }),
+            Action::TogglePanePinned { .. } => Ok(ProtobufAction {
+                name: ProtobufActionName::TogglePanePinned as i32,
+                optional_payload: None,
+            }),
+            Action::TogglePaneInGroup { .. } => Ok(ProtobufAction {
+                name: ProtobufActionName::TogglePaneInGroup as i32,
+                optional_payload: None,
+            }),
+            Action::ToggleGroupMarking { .. } => Ok(ProtobufAction {
+                name: ProtobufActionName::ToggleGroupMarking as i32,
+                optional_payload: None,
+            }),
+            Action::NewStackedPane(..) => Ok(ProtobufAction {
+                name: ProtobufActionName::NewStackedPane as i32,
+                optional_payload: None,
+            }),
             Action::NoOp
             | Action::Confirm
             | Action::NewInPlacePane(..)
@@ -1293,6 +1271,8 @@ impl TryFrom<Action> for ProtobufAction {
             | Action::DumpLayout
             | Action::CliPipe { .. }
             | Action::ListClients
+            | Action::StackPanes(..)
+            | Action::ChangeFloatingPaneCoordinates(..)
             | Action::SkipConfirm(..) => Err("Unsupported action"),
         }
     }
@@ -1433,6 +1413,51 @@ impl TryFrom<Position> for ProtobufPosition {
     }
 }
 
+impl TryFrom<ProtobufMouseEventPayload> for MouseEvent {
+    type Error = &'static str;
+    fn try_from(protobuf_event: ProtobufMouseEventPayload) -> Result<Self, &'static str> {
+        Ok(MouseEvent {
+            event_type: match protobuf_event.event_type as u32 {
+                0 => MouseEventType::Press,
+                1 => MouseEventType::Release,
+                _ => MouseEventType::Motion,
+            },
+            left: protobuf_event.left as bool,
+            right: protobuf_event.right as bool,
+            middle: protobuf_event.middle as bool,
+            wheel_up: protobuf_event.wheel_up as bool,
+            wheel_down: protobuf_event.wheel_down as bool,
+            shift: protobuf_event.shift as bool,
+            alt: protobuf_event.alt as bool,
+            ctrl: protobuf_event.ctrl as bool,
+            position: Position::new(protobuf_event.line as i32, protobuf_event.column as u16),
+        })
+    }
+}
+
+impl TryFrom<MouseEvent> for ProtobufMouseEventPayload {
+    type Error = &'static str;
+    fn try_from(event: MouseEvent) -> Result<Self, &'static str> {
+        Ok(ProtobufMouseEventPayload {
+            event_type: match event.event_type {
+                MouseEventType::Press => 0,
+                MouseEventType::Release => 1,
+                MouseEventType::Motion => 2,
+            } as u32,
+            left: event.left as bool,
+            right: event.right as bool,
+            middle: event.middle as bool,
+            wheel_up: event.wheel_up as bool,
+            wheel_down: event.wheel_down as bool,
+            shift: event.shift as bool,
+            alt: event.alt as bool,
+            ctrl: event.ctrl as bool,
+            line: event.position.line.0 as i64,
+            column: event.position.column.0 as i64,
+        })
+    }
+}
+
 impl TryFrom<ProtobufPluginConfiguration> for PluginUserConfiguration {
     type Error = &'static str;
     fn try_from(plugin_configuration: ProtobufPluginConfiguration) -> Result<Self, &'static str> {
